/* ----------------------------------------------------------------------- *
 *   
 *   Copyright 2001 H. Peter Anvin - All Rights Reserved
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, Inc., 675 Mass Ave, Cambridge MA 02139,
 *   USA; either version 2 of the License, or (at your option) any later
 *   version; incorporated herein by reference.
 *
 * ----------------------------------------------------------------------- */

/*
 * Character type and conversions package for MOO
 */

#include <stddef.h>
#include <ctype.h>
#include <limits.h>
#include <errno.h>
#include <float.h>
#include <time.h>
#include <sys/time.h>
#include "my-math.h"
#include "my-stdlib.h"
#include "my-string.h"
#include "my-time.h"

#include "config.h"
#include "functions.h"
#include "log.h"
#include "random.h"
#include "storage.h"
#include "structures.h"
#include "utils.h"

/**** built in functions ****/

#define CISFUNC(F) \
static package \
bf_ ## F (Var arglist, Byte next, void *vdata, Objid progr) \
{ \
  Var r; \
  unsigned char ch; \
  \
  ch = arglist.v.list[1].v.str[0]; \
  \
  free_var(arglist); \
  \
  if ( !ch ) { \
    return make_error_pack(E_RANGE); /* Empty string */ \
  } else { \
    r.type = TYPE_INT; \
    r.v.num = F (ch) ? 1 : 0; \
    return make_var_pack(r); \
  } \
}

CISFUNC(isupper)
CISFUNC(islower)
CISFUNC(isalpha)
CISFUNC(isdigit)
CISFUNC(isxdigit)
CISFUNC(isspace)
CISFUNC(isprint)
CISFUNC(isgraph)
CISFUNC(iscntrl)
CISFUNC(ispunct)
CISFUNC(isalnum)
CISFUNC(isascii)

static package
bf_ord(Var arglist, Byte next, void *vdata, Objid progr)
{
  Var r;
  unsigned char ch;

  ch = arglist.v.list[1].v.str[0];

  free_var(arglist);

  if ( !ch ) {
    return make_error_pack(E_RANGE); /* Empty string */
  } else {
    r.type = TYPE_INT;
    r.v.num = ch;
    return make_var_pack(r);
  }
}

static package
bf_chr(Var arglist, Byte next, void *vdata, Objid progr)
{
  int n = arglist.v.list[0].v.num;
  int i;
  Var r;
  int32 ch;
  unsigned char *p;
  enum error e = E_NONE;

  r.type = TYPE_STR;
  r.v.str = p = mymalloc(sizeof(char) * (n+1), M_STRING);

  for ( i = 1 ; i <= n ; i++ ) {
    if ( arglist.v.list[i].type != TYPE_INT ) {
      e = E_TYPE;
      break;
    }
    ch = arglist.v.list[i].v.num;

    if ( ch < 0 || ch > 255 || !(ch == ' ' || ch == '\t' || isgraph(ch)) ) {
      e = E_INVARG;		/* Invalid character code */
      break;
    } else {
      *p++ = ch;
    }
  }

  free_var(arglist);

  if ( e != E_NONE ) {
    myfree((char *)r.v.str, M_STRING);
    return make_error_pack(e);
  } else {
    *p = '\0';			/* Null-terminate string */
    return make_var_pack(r);
  }
}

static package
bf_toupper(Var arglist, Byte next, void *vdata, Objid progr)
{
  Var r;
  unsigned char *p;

  r.type = TYPE_STR;
  r.v.str = str_dup(arglist.v.list[1].v.str);
  p = (unsigned char *)r.v.str;

  free_var(arglist);

  while ( *p ) {
    *p++ = toupper(*p);
  }

  return make_var_pack(r);
}

static package
bf_tolower(Var arglist, Byte next, void *vdata, Objid progr)
{
  Var r;
  unsigned char *p;

  r.type = TYPE_STR;
  r.v.str = str_dup(arglist.v.list[1].v.str);
  p = (unsigned char *)r.v.str;

  free_var(arglist);

  while ( *p ) {
    *p++ = tolower(*p);
  }

  return make_var_pack(r);
}

void
register_charconv(void)
{
  register_function("isupper", 1, 1, bf_isupper, TYPE_STR);
  register_function("islower", 1, 1, bf_islower, TYPE_STR);
  register_function("isalpha", 1, 1, bf_isalpha, TYPE_STR);
  register_function("isdigit", 1, 1, bf_isdigit, TYPE_STR);
  register_function("isxdigit", 1, 1, bf_isxdigit, TYPE_STR);
  register_function("isspace", 1, 1, bf_isspace, TYPE_STR);
  register_function("isprint", 1, 1, bf_isprint, TYPE_STR);
  register_function("isgraph", 1, 1, bf_isgraph, TYPE_STR);
  register_function("iscntrl", 1, 1, bf_iscntrl, TYPE_STR);
  register_function("ispunct", 1, 1, bf_ispunct, TYPE_STR);
  register_function("isalnum", 1, 1, bf_isalnum, TYPE_STR);
  register_function("isascii", 1, 1, bf_isascii, TYPE_STR);
  
  register_function("ord", 1, 1, bf_ord, TYPE_STR);
  register_function("chr", 0, -1, bf_chr);
  register_function("toupper", 1, 1, bf_toupper, TYPE_STR);
  register_function("tolower", 1, 1, bf_tolower, TYPE_STR);
}

char rcsid_charconv[] = "$Id$";

/* 
 * $Log$
 */
